import bpy
import os
from bpy.types import Operator
from bpy import context
import addon_utils
from pathlib import Path


class RAIN_OP_add(Operator):
    """ Add Rain """
    bl_idname = "bagarain.add_rain"
    bl_label = 'Add Rain'
    bl_options = {'REGISTER', 'UNDO'}

    @classmethod
    def poll(cls, context):
        o = context.object
        
        if len(bpy.context.selected_objects) > 0 and o.type == 'MESH':
                    return context.selected_objects

    def execute(self, context):
        rain_pref = context.preferences.addons[__package__].preferences

    ###################################################################################
    # SETUP COLLECTION
    ###################################################################################
        # if rain mail coll is already present
        if bpy.data.collections.get("Baga Rain Generator") is not None:
            main_coll = bpy.data.collections["Baga Rain Generator"]
            rain_coll = bpy.data.collections.new("BRG_Rain") # RAIN COLL
            main_coll.children.link(rain_coll)

            target_coll = bpy.data.collections.new("BRG_Targets") # Targets
            rain_coll.children.link(target_coll)

        else:
            coll_targets = Collection_Add(self,context,"BRG_Rain","BRG_Targets")
            rain_coll = coll_targets[0]
            target_coll = coll_targets[1]


    ###################################################################################
    # ADD RAIN
    ###################################################################################

        rain_sim_mesh = bpy.data.meshes.new('BRG_Rain_Simulation')
        rain_sim_obj = bpy.data.objects.new("BRG_Rain_Simulation", rain_sim_mesh)
        rain_coll.objects.link(rain_sim_obj)

        # if rain_pref.rain_source_mode == False:
        #     targets = bpy.context.selected_objects
        #     for target in targets:
        #         target.select_set(True)
        #         target_coll.objects.link(target)
                
        targets = bpy.context.selected_objects
        for target in targets:
            target.select_set(True)
            if target.type == 'MESH' or target.type == 'CURVE':
                target_coll.objects.link(target)



    ###################################################################################
    # ADD MODIFIER
    ###################################################################################           
        new = rain_sim_obj.modifiers.new
        modifier = new(name="Baga_Rain_Generator", type='NODES')


    ###################################################################################
    # IMPORT AND SETUP IVY NODES
    ###################################################################################
        Add_NodeGroup(self,context,modifier)

        # SETUP COLLECTIONS
        modifier["Input_8"] = target_coll # TARGET

        # GEONODES UI MODIFIER NAMES
        modifier["Input_9"] = "MAIN SETTINGS"
        modifier["Input_10"] = "DROPLETS"
        modifier["Input_12"] = "SPLASHES"
        modifier["Input_20"] = "WIND"
        modifier["Input_43"] = "VISIBILITY"
        modifier["Input_51"] = "LOOP"

        # NODE TREE VALUES
        modifier["Input_14"] = rain_pref.rain_distribution_method
        modifier["Input_26"] = rain_pref.splash_density_viewport
        modifier["Input_11"] = rain_pref.splash_density_render
        modifier["Input_29"] = rain_pref.splash_radius_viewport
        modifier["Input_28"] = rain_pref.splash_radius_render
        modifier["Input_2"] = rain_pref.rain_density
        modifier["Input_40"] = rain_pref.rain_gravity
        modifier["Input_4"] = rain_pref.rain_altitude
        modifier["Input_3"] = rain_pref.rain_speed
        modifier["Input_5"] = rain_pref.splash_speed

        modifier["Input_17"][0] = rain_pref.droplet_scale[0]
        modifier["Input_17"][1] = rain_pref.droplet_scale[1]
        modifier["Input_17"][2] = rain_pref.droplet_scale[2]
        modifier["Input_48"] = rain_pref.pers_bouncing
        modifier["Input_37"] = rain_pref.copl_bouncing
        modifier["Input_41"] = rain_pref.randomize_impact
        modifier["Input_38"] = rain_pref.splash_shape


    ###################################################################################
    # LAST STEP
    ###################################################################################
        bpy.ops.object.select_all(action='DESELECT')
        bpy.context.view_layer.objects.active = rain_sim_obj
        rain_sim_obj.select_set(True)

        return {"FINISHED"}



###################################################################################
# ADD NODEGROUP TO THE MODIFIER
###################################################################################
def Add_NodeGroup(self,context,modifier):
    try:
        modifier.node_group = bpy.data.node_groups["Baga_Rain_Generator"]
    except:
        Import_Nodes(self,context)
        modifier.node_group = bpy.data.node_groups["Baga_Rain_Generator"]

###################################################################################
# IMPORT NODE GROUP
###################################################################################
def Import_Nodes(self,context):

    # for mod in addon_utils.modules():
    #     if mod.bl_info['name'] == __package__:
    #         filepath = mod.__file__
    #         file_path = filepath.replace("__init__.py","Baga_Rain_Generator_NodeTree_V10.blend")
    #     else: pass
    file_path = os.path.join(os.path.dirname(__file__),"Baga_Rain_Generator_NodeTree_V10.blend")
    inner_path = "NodeTree"
    nodes_name = "Baga_Rain_Generator"

    bpy.ops.wm.append(
        filepath = os.path.join(file_path, inner_path, nodes_name),
        directory = os.path.join(file_path, inner_path),
        filename=nodes_name
        )

###################################################################################
# MANAGE COLLECTION
###################################################################################
def Collection_Add(self,context,rain_coll_name,sub_coll_name):

    # Create collection and check if the main "Baga Collection" does not already exist
    if bpy.data.collections.get("Baga Rain Generator") is None:
        main_coll = bpy.data.collections.new("Baga Rain Generator") # MAIN
        bpy.context.scene.collection.children.link(main_coll)
        if bpy.data.collections.get(rain_coll_name) is None:
            rain_coll = bpy.data.collections.new(rain_coll_name) # RAIN COLL
            main_coll.children.link(rain_coll)
        else:
            rain_coll = bpy.data.collections.get(rain_coll_name)
        sub_coll = bpy.data.collections.new(sub_coll_name) # SUB CATEGORY COLL
        rain_coll.children.link(sub_coll)

    # If the main collection Baga Rain already exist
    elif bpy.data.collections.get(sub_coll_name) is None:
        main_coll = bpy.data.collections["Baga Rain Generator"]
        if bpy.data.collections.get(rain_coll_name) is None:
            rain_coll = bpy.data.collections.new(rain_coll_name) # RAIN COLL
            main_coll.children.link(rain_coll)
        else:
            rain_coll = bpy.data.collections.get(rain_coll_name)
        sub_coll = bpy.data.collections.new(sub_coll_name) # SUB CATEGORY COLL
        rain_coll.children.link(sub_coll)

    # If the main collection Baga Rain already exist
    else:
        rain_coll = bpy.data.collections.get(rain_coll_name)
        sub_coll = bpy.data.collections.get(sub_coll_name)

    return rain_coll, sub_coll

###################################################################################
# DISPLAY WARNING
###################################################################################
def Warning(message = "", title = "Message Box", icon = 'INFO'):

    def draw(self, context):
        self.layout.label(text=message)
    bpy.context.window_manager.popup_menu(draw, title = title, icon = icon)
