import bpy
from bpy.types import Operator
from . import rain_icons

class RAIN_PT_Panel(bpy.types.Panel):
    """Creates a Panel in the Object properties window"""
    bl_idname = "RAIN_PT_Panel"
    bl_space_type = 'VIEW_3D'
    bl_region_type = 'UI'
    bl_category = "BagaRain"
    bl_label = "Baga Rain Generator"

    # panel_sub_method: bpy.props.BoolProperty(name="Computing Methods", default=False)

    def draw(self, context):
        layout = self.layout
        col = layout.column()
        rain_pref = context.preferences.addons[__package__].preferences
        obj = context.object

        ###################################################################################
        # IF RAIN IS SELECTED
        ###################################################################################  
        if obj is not None and obj.name.startswith("BRG_"):
            rain_modifier = obj.modifiers["Baga_Rain_Generator"]
            col = layout.column(align=False)



        # MAIN SETTINGS
            if rain_pref.rain_main_settings == False:
                col.separator(factor = 0.5)
                col.prop(rain_pref, 'rain_main_settings', text = "Main Settings", emboss = False)
                col.separator(factor = 1.5)
            else:
                box = col.box()
                box.prop(rain_pref, 'rain_main_settings', text = "Main Settings", emboss = False)
                col.separator(factor = 1.5)
                box.label(text="Animation Mode")
                Double_Row_Switch(box, rain_modifier, "Input_53", "Continue", "Loop")
                col_a = box.column(align = True)
                col_a.prop(rain_modifier, '["Input_2"]', text="Rain Density")
                if rain_modifier["Input_53"]:
                    col_a.prop(rain_modifier, '["Input_52"]', text="Speed")
                    col_a.prop(rain_modifier, '["Input_50"]', text="Frame End Loop")
                    col_a.prop(rain_modifier, '["Input_54"]', text="Frame Offset Loop")
                    tips = col_a.operator("bagarain.tooltips", text="", depress = False, icon = 'INFO')
                    tips.message = 'The loop mode does not involve the shader, as it is unfortunately not available in loop mode. Slow motion is also limited: a water droplet needs to fall and fully explode to complete a loop, so consider reducing the Altitude to shorten its path.'
                    tips.title = "Loop"
                    tips.image = "None"
                    tips.scale = 8
                    tips.size = 50
                    tips.url = "None"
                else:
                    col_a.prop(rain_modifier, '["Input_3"]', text="Speed")

                # ui_type, modifier, index, name_a, name_b
                box.label(text="Distribution Method :")
                Double_Row_Switch(box, rain_modifier, "Input_14", "Accurate", "Large Scene")
                if rain_modifier["Input_14"] == False:
                    # accurate
                    tips = box.operator("bagarain.tooltips", text="", depress = False, icon = 'INFO')
                    tips.message = 'Rain distribution using the Accurate method: the density of the rain is based on the total surface area of what is targeted by the rain. This method is recommended for smaller scenes where objects assigned as targets overlap.'
                    tips.title = "Accurate Method"
                    tips.image = "accurate"
                    tips.scale = 8
                    tips.size = 50
                    tips.url = "None"
                else:
                    # large area
                    tips = box.operator("bagarain.tooltips", text="", depress = False, icon = 'INFO')
                    tips.message = 'Rain distribution using the Large Scene method: the density of the rain is calculated for each object. If two objects overlap, then the amount of rain also overlaps. This method is useful if your surface is divided into multiple islands and separated by long distances.'
                    tips.title = "Large Scene Method"
                    tips.image = "large_area"
                    tips.scale = 8
                    tips.size = 50
                    tips.url = "None"

                box.label(text="Material :")
                box.prop_search(rain_modifier, '["Input_30"]', bpy.data, "materials", text="", icon="MATERIAL")


        # DROPLETS
            if rain_pref.rain_droplets_settings == False:
                col.separator(factor = 0.5)
                col.prop(rain_pref, 'rain_droplets_settings', text = "Droplets", emboss = False)
                col.separator(factor = 1.5)
            else:
                box = col.box()
                box.prop(rain_pref, 'rain_droplets_settings', text = "Droplets", emboss = False)
                col.separator(factor = 1.5)
                box.label(text="Droplets")
                row = box.row(align=True)
                row.prop(rain_modifier, '["Input_4"]', text="Altitude")
                tips = row.operator("bagarain.tooltips", text="", depress = False, icon = 'INFO')
                tips.message = 'Altitude affects the speed, so you may need to readjust the rain speed, making it one of the first parameters to adjust. Note that in Loop mode, reducing the altitude allows for shorter loops.'
                tips.title = "Altitude"
                tips.image = "None"
                tips.scale = 8
                tips.size = 50
                tips.url = "None"

                # ui_type, modifier, index, name_a, name_b
                Double_Row_Switch(box, rain_modifier, "Input_13", "Point", "Droplets")
                tips = box.operator("bagarain.tooltips", text="", depress = False, icon = 'INFO')
                tips.message = 'For performance reasons, keep Points in the viewport and enable Render as Droplets. Be aware that Motion Blur can significantly increase render time. We suggest rendering the rain without motion blur in a separate layer if you still wish to use motion blur in your scene.'
                tips.title = "Viewport Performances"
                tips.image = "None"
                tips.scale = 8
                tips.size = 50
                tips.url = "None"

                col_b = box.column(align=True)
                if rain_modifier["Input_13"] == False:
                    col_b.label(text="Visibility :")
                    col_b.operator('switch.buttonrain', depress = rain_modifier["Input_15"], text = "Render as Droplets").index = "Input_15"
                    if rain_modifier["Input_15"] == True:
                        col_b.prop(rain_modifier, '["Input_16"]', text="Random Length")
                        row_b = col_b.row(align=True)
                        row_b.label(text="Scale :")
                        col_b_b = row_b.column()
                        col_b_b.prop(rain_modifier, '["Input_17"]', text="")
                else:
                    col_b.prop(rain_modifier, '["Input_16"]', text="Random Length")
                    row_b = col_b.row(align=True)
                    row_b.label(text="Scale :")
                    col_b_b = row_b.column()
                    col_b_b.prop(rain_modifier, '["Input_17"]', text="")

        
        # SPLASHES
            if rain_pref.rain_splashes_settings == False:
                col.separator(factor = 0.5)
                col.prop(rain_pref, 'rain_splashes_settings', text = "Splashes", emboss = False)
                col.separator(factor = 1.5)
            else:
                box = col.box()
                box.prop(rain_pref, 'rain_splashes_settings', text = "Splashes", emboss = False)
                col.separator(factor = 1.5)
                box.prop(rain_modifier, '["Input_5"]', text="Speed")
                box.prop(rain_modifier, '["Input_40"]', text="Gravity")

                box.label(text="Visibility")
                row_vis = box.row(align=True)
                if rain_modifier["Input_46"] == True:
                    spl_render_icon = 'RESTRICT_RENDER_OFF'
                else:
                    spl_render_icon = 'RESTRICT_RENDER_ON'
                if rain_modifier["Input_47"] == True:
                    spl_viewport_icon = 'RESTRICT_VIEW_OFF'
                else:
                    spl_viewport_icon = 'RESTRICT_VIEW_ON'
                row_vis.operator('switch.buttonrain', depress = rain_modifier["Input_46"], text = "Render", icon =spl_render_icon).index = "Input_46"
                row_vis.operator('switch.buttonrain', depress = rain_modifier["Input_47"], text = "Viewport", icon =spl_viewport_icon).index = "Input_47"
                
                box.label(text="Radius")
                coldens = box.column(align=True)
                coldens.prop(rain_modifier, '["Input_29"]', text="Viewport")
                coldens.prop(rain_modifier, '["Input_28"]', text="Render")
                tips = coldens.operator("bagarain.tooltips", text="", depress = False, icon = 'INFO')
                tips.message = 'Radius of the water droplets that make up the splashes. From the impact, the droplets become smaller and smaller until they disappear. It may be necessary to increase the transparency ( Light Paths > Max Bounces > Transparent) to prevent the overlapping particles/splashes from being black.'
                tips.title = "Splashes Radius"
                tips.image = "splash"
                tips.scale = 8
                tips.size = 50
                tips.url = "https://youtu.be/iGB_JC_tpkM"

                box.label(text="Density")
                coldens = box.column(align=True)
                coldens.prop(rain_modifier, '["Input_26"]', text="Viewport")
                coldens.prop(rain_modifier, '["Input_11"]', text="Render")

                box.label(text="Surface Bouncing")
                colsp = box.column(align=True)
                colsp.prop(rain_modifier, '["Input_48"]', text="Perpendicular")
                colsp.prop(rain_modifier, '["Input_37"]', text="Coplanar")
                tips = colsp.operator("bagarain.tooltips", text="", depress = False, icon = 'INFO')
                tips.message = 'The bounce upon impact can be adjusted differently depending on the angle of impact on the surface. These two parameters replace the "height" parameter from the previous version.'
                tips.title = "Splashes Bouncing"
                tips.image = "splash"
                tips.scale = 8
                tips.size = 50
                tips.url = "https://youtu.be/3WYmS9pPu08"
                
                box.label(text="Dispersion")
                coldi = box.column(align=True)
                coldi.prop(rain_modifier, '["Input_36"]', text="Radius")
                coldi.prop(rain_modifier, '["Input_39"]', text="Scale")
                coldi.prop(rain_modifier, '["Input_41"]', text="Randomize")
                coldi.prop(rain_modifier, '["Input_38"]', text="Shape")

        
        # WIND
            if rain_pref.rain_wind_settings == False:
                col.separator(factor = 0.5)
                col.prop(rain_pref, 'rain_wind_settings', text = "Wind", emboss = False)
                col.separator(factor = 1.5)
            else:
                box = col.box()
                box.prop(rain_pref, 'rain_wind_settings', text = "Wind", emboss = False)
                col.separator(factor = 1.5)
                box.prop(rain_modifier, '["Input_18"]', text="Oriantation")
                box.prop(rain_modifier, '["Input_19"]', text="Angle")
                
                tips = box.operator("bagarain.tooltips", text="", depress = False, icon = 'INFO')
                tips.message = 'Define the wind direction here. Use the Angle to adjust the "intensity" of the wind and then the direction. Be careful on certain complex surfaces, you may need to adjust the Splashes after orienting the wind.'
                tips.title = "Wind"
                tips.image = "wind"
                tips.scale = 8
                tips.size = 50
                tips.url = "https://youtu.be/LfKKws5VsWU"


        # TARGET
            if rain_pref.rain_target_settings == False:
                col.separator(factor = 0.5)
                col.prop(rain_pref, 'rain_target_settings', text = "Targets", emboss = False)
                col.separator(factor = 1.5)
            else:
                box = col.box()
                box.prop(rain_pref, 'rain_target_settings', text = "Targets", emboss = False)
                col.separator(factor = 1.5)
                row = box.row(align = True)
                row.scale_y = 1.8
                add = row.operator("rainadd.target", text= "Add", icon = 'ADD')
                add.index = 8
                remove = row.operator("rainremove.target", text= "Remove", icon = 'REMOVE')
                remove.index = 8

                box.label(text="Target Collection :")
                box.prop_search(rain_modifier, '["Input_8"]', bpy.data, "collections", text="", icon="OUTLINER_COLLECTION")


        # VISIBILITY
            row_view = col.row(align=True)
            if rain_modifier["Input_42"] == True:
                view_render_icon = 'RESTRICT_RENDER_OFF'
            else:
                view_render_icon = 'RESTRICT_RENDER_ON'
            if rain_modifier["Input_45"] == True:
                view_viewport_icon = 'RESTRICT_VIEW_ON'
            else:
                view_viewport_icon = 'RESTRICT_VIEW_OFF'
            row_view.operator('switch.buttonrain', depress = not rain_modifier["Input_45"], text = "Viewport", icon=view_viewport_icon).index = "Input_45"
            row_view.operator('switch.buttonrain', depress = rain_modifier["Input_42"], text = "Render", icon=view_render_icon).index = "Input_42"



        # DELETE
            if bpy.context.object.mode == 'OBJECT':
                col_end=col.column(align=True)
                col_end.alert = True
                col_end.operator("rain.delete", text= "Delete Rain", icon = 'TRASH')


        # WARNING EEVEE / MOTION BLUR
            if bpy.context.scene.render.engine == 'BLENDER_EEVEE':
                box_alert = layout.box()
                box_alert.alert = True
                box_alert.label(text="Splashes unavailable EEVEE")
                box_alert.label(text="Switch to Cycles")
            elif bpy.context.scene.render.use_motion_blur == True:
                box_alert = layout.box()
                box_alert.alert = True
                box_alert.label(text="Disable motion blur,")
                box_alert.label(text="to save rendering time.")


        ###################################################################################
        # IF RAIN IS NONE
        ###################################################################################  
        elif obj is not None and obj.type == 'MESH':
            col = layout.column(align=False)

            # RAIN GENERATOR
            row = col.row(align = True)
            row.scale_y = 3
            row.operator("bagarain.add_rain", text="Add Rain !", depress = False, icon = 'ADD')            
            
            tips = row.operator("bagarain.tooltips", text="", depress = False, icon = 'INFO')
            tips.message = 'Select all the objects that should be affected by the rain. They will be linked to a new collection called Targets, and if you wish to add or remove objects from this collection, please use the dedicated section in this panel. You can add as many different rain generators as you want.'
            tips.title = "Adding Rain Generator"
            tips.image = "addrain_icon"
            tips.scale = 8
            tips.size = 50
            tips.url = "https://youtu.be/oTYbld4-U9E"

            # SHADER
            row = col.row(align = True)
            row.scale_y = 2
            row.operator("bagarain.add_rain_shader", text="Add Rain Shader", depress = False, icon = 'MATERIAL')
            
            tips = row.operator("bagarain.tooltips", text="", depress = False, icon = 'INFO')
            tips.message = 'This option adds a node group into each materials of the selected object. This node adds a rain shader to your meterial, you can tweak it in the shader editor. Beware, this may increase the compilation time. You can of course remove this node at any time in the shader editor. Loop option not available.'
            tips.title = "Adding Rain Shader"
            tips.image = "rainshader_icon"
            tips.scale = 8
            tips.size = 50
            tips.url = "https://youtu.be/0tgvoG_Oz9c"

        elif obj is not None:
            box = layout.box()
            box.label(text="Select a Mesh")

        ###################################################################################
        # ALPHA WARNING
        ###################################################################################
        else:
            box = layout.box()
            box.label(text="Select Target(s)")


###################################################################################
# DISPLAY TOOLTIPS
###################################################################################
class RAIN_tooltips(Operator):
    """Display a tooltips"""
    bl_idname = "bagarain.tooltips"
    bl_label = "Tips"

    message: bpy.props.StringProperty(default="None")
    title: bpy.props.StringProperty(default="Tooltip")
    icon: bpy.props.StringProperty(default="INFO")
    image: bpy.props.StringProperty(default="None")
    scale: bpy.props.FloatProperty(default=5)
    size: bpy.props.IntProperty(default=50)
    url: bpy.props.StringProperty(default="None")

    def execute(self, context):
        Tooltip(self.message, self.title, self.icon, self.image, self.scale, self.size, self.url) 
        return {'FINISHED'}

def Tooltip(message = "", title = "Message Box", icon = 'INFO', image = "None", scale = 5, size = 50, url = "None"):

    def draw(self, context):
        layout = self.layout
        col = layout.column()
        if image != "None":
            col.template_icon(icon_value = rain_icons.previews[image].icon_id, scale=scale)
        # DIRTY LAZY CODE START
        count = 0
        mess = message
        length = int(size)
        caracter = length
        temp = 0
        for i in message:            
            if count == 0:
                if mess[0] == " ":
                    o = length
                    if len(mess) > o:
                        while mess[o] != " ":
                            o += 1
                            if len(mess) == o:
                                break
                    col.label(text=mess[1:o])
                    caracter = length

                elif mess == message:
                    o = length
                    if o >= len(mess):
                        o = len(mess)-1
                    while message[o] != " ":
                        o += 1
                        if len(mess) == o:
                            break
                    col.label(text=mess[0:o])
                    caracter = length                    
                else :
                    count = temp
                    caracter += 1
            count += 1
            temp = count
            mess = mess[1:]
            if count == caracter:
                count = 0
        # DIRTY LAZY CODE END
        if url != "None":
            col.separator(factor = 1.5)
            row = col.row()
            row.scale_y = 2
            row.operator("wm.url_open", text="Video Demo", icon = 'PLAY', depress = False).url = url


    bpy.context.window_manager.popup_menu(draw, title = title, icon = icon)


###################################################################################
# Switch row
###################################################################################
def Double_Row_Switch(ui_type = '', modifier = '', index = '', name_a ='', name_b=''):
    row = ui_type.row(align = True)
    row.operator('switch.buttonrain', depress = not modifier[index], text = name_a).index = index
    row.operator('switch.buttonrain', depress = modifier[index], text = name_b).index = index